/*--------------------------------------------------------------------------
  CRP_A10
  Implement A10.1 in CreditRisk+ manual to calculate loss distribution.
  Returns only selected quantiles of distribution.

  Use symbolic constant CDFPREC to control precision of internal
  calculations.  Currently set to double.

  Michael Gordy
  With thanks to Gary Anderson
  Version 1.02   April 6, 2000
--------------------------------------------------------------------------*/

#include <math.h>
#include "mex.h"
#define LNCMIN -250
#define CDFPREC double

/*---------------------------------------------------------------------
  CRPPOLY:  Here is where all the work gets done.
    Nothing tricky in the implementation of the recurrence relation.
    However, we need to do a bit of re-scaling of the probabilities
    in order to avoid underflows.  We set symbolic constant LNCMIN
    to be the smallest value of log(G(0)) which would be tolerable
    without re-scaling. If lnc0 (==log(G(0))) comes in smaller,
    then we use mgma and gma to keep track of the re-scaling, so
    we can scale back up at the end.
  ----------------------------------------------------------------------*/
int crppoly(double *pctiles, double *moments,
             double *A, int r, double *B, int s, double lnc0, 
             double *pctknots, int numknots, int nmax, 
             double nearone, double lossunit) {
  int n=0,i,endcode,k=0,Csize;
  double vk;
  CDFPREC *C, G, rhs, gma=1.0, mgma=1.0;  
  /* initialize lossunit variables for lnc0 */
  if (lnc0<LNCMIN) {
    mgma=lnc0/(lnc0-LNCMIN);
    gma=exp(LNCMIN);
  }
  nearone=nearone/gma;
  /* initialize moment and pctiles vector */
  for (i=0; i<4; i++)
    *(moments+i)=0.0;
  for (i=0; i<numknots; i++)
    *(pctiles+i)=0.0;
  /* We only need to carry forward max(1+r,s) units of C */
  Csize=(s>r) ? s+3 : r+4;      /* but leave a little leeway */
  C=(CDFPREC *)mxCalloc(Csize,sizeof(CDFPREC));
  /* Prob x=0 is G=c0 for n=0. */
  *C=exp(lnc0/mgma); G=*C; 
  while (G<nearone & n<nmax & *(C+(n % Csize))>=0) { 
    rhs=0.0;
    for (i=0;(i<=r && i<=n);i++) 
      rhs+=(*(A+i))*(*(C+((n-i+Csize) % Csize)));
    if (n>0) 
      for (i=0;(i<s && i<n);i++) 
        rhs-=(*(B+i+1))*(n-i)*(*(C+((n-i+Csize) % Csize)));
    *(C+((n+1) % Csize))=rhs/((n+1)*(*B));  	
    n++;     /* increment n */    
    /* add to moments vector */
    for (vk=1.0, i=0;i<4;i++) {
      vk*=(n*lossunit);
      *(moments+i)+=vk*(*(C+(n % Csize)));
      }
    G+=(*(C+(n % Csize)));   /* tally into cumulative */
    /* check on pctiles vector */
    while ((k<numknots) && (G>=(*(pctknots+k))/gma)) {
      *(pctiles+k)=n*lossunit; k++;
      }
    } 
  /* Re-scale moments */
  for (i=0;i<4;i++)
      *(moments+i)*=gma;
  if (*(C+(n % Csize))<0)
     endcode=1;
  else if (n==nmax)
     endcode=2;
  else
     endcode=0;
  mxFree(C);
  return endcode;
}


/*--------------------------------mexFunction------------------------------
  mexFunction is the usual gateway routine for compilation as a mex file.
  No mysteries here!
  Arguments mapped to (A,B,lnc0,pctknots,nmax,nearone,lossunit).
  --------------------------------mexFunction------------------------------*/
void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray *prhs[]) {
  double *A,*B,*pctknots,*moments,*pctiles,lnc0,dnmax,nearone,*passcode,lossunit;
  int r,s,i,nmax,numknots,endcode;
  if (nrhs!=7)
    mexErrMsgTxt("Incorrect number of parameters. Need seven.");
  A=mxGetPr(prhs[0]);
  B=mxGetPr(prhs[1]);
  r=mxGetN(prhs[0])-1;
  s=mxGetN(prhs[1])-1;
  lnc0=mxGetScalar(prhs[2]);
  pctknots=mxGetPr(prhs[3]);
  numknots=mxGetM(prhs[3]);
  dnmax=mxGetScalar(prhs[4]);
  nmax=(int) dnmax;
  nearone=mxGetScalar(prhs[5]);
  lossunit=mxGetScalar(prhs[6]);
  plhs[0]=mxCreateDoubleMatrix(4,1,mxREAL);
  plhs[1]=mxCreateDoubleMatrix(numknots,1,mxREAL);
  plhs[2]=mxCreateDoubleMatrix(1,1,mxREAL);
  moments=mxGetPr(plhs[0]);
  pctiles=mxGetPr(plhs[1]);
  passcode=mxGetPr(plhs[2]);
  endcode=crppoly(pctiles,moments,A,r,B,s,lnc0,pctknots,
                       numknots,nmax,nearone,lossunit);
  *passcode=(double) endcode;
}
