function kappa=crpcumulants(defprob,lossvalue,w,sigma,R)
%CRPCUMULANTS:  Cumulants in CreditRisk+
% kappa=crpcumulants(defprob,lossvalue,w,sigma,R)
% Algorithm for direct calculation of CR+ cumulants described in
% paper by M. Gordy, "Calculation of Higher Moments in CreditRisk+
% with Applications."
%
% Inputs:
%    defprob   = unconditional default probabilities for obligors
%                Nx1 or scalar (if scalar, assume same for all obligors)
%    lossvalue = book value loss in event of default
%                Nx1 vector (full vector required!!)
%    w     = factor loadings on systematic factors
%            NxK or 1xK (if row vector, assume same for all obligors)
%    sigma = sector volatilities: sigma(k) is standard deviation of X_k
%            1xK vector
%    R     = number of cumulants to calculate (up to 6)
%            (optional: default value is 4)
%
% Outputs:
%    kappa = analytical cumulants of the loss distribution (1xR)
%
% Notes:
% * This implementation covers only the first 6 cumulants.
% * This method does NOT require loss exposures to be expressed as
%   integer multiples of standardized loss unit.  Formula is exact,
%   with no loss of accuracy or speed, whether losses are rounded
%   or not.
% * Formula for conditional default probability for obligor i is
%      p_i(x) = DefProb_i * (1+ sum_k w_{ik}*(x_k+1))
%                 where underscore indicates subscript 
%                 and summation is for k=1..K
%      Factor loadings w_{ik} sum across k to less than or equal to
%      one.  If less than one, remaining risk is assigned to
%      specific risk sector (see A12.3 in CR+ manual).
% * Distribution for X_k is gamma with mean 1, variance sigma_k^2.
%      NOTE!  The CR+ manual has mean of X_k equal to mu_k, but
%      then normalizes out the mu_k in calculating the unconditional
%      loss distribution.  My approach is numerically equivalent.
%
% Requires:  expandcrpinputs.m

% CreditRisk+ is trademark of Credit Suisse Financial Products.
% See CSFP (1997) "CreditRisk+: A Credit Risk Management Framework."
%
% Version 1.02 of Matlab implementation by Michael Gordy, 7 March 2000.
% Email: <michael.gordy@frb.gov>
% This program is provided free of charge for research use.  It
% does not constitute endorsement of CreditRisk+ by the Federal
% Reserve Board.

% Expand data, as some vectors may have been given as scalars.
[defprob,w,sigma,N,K,K1]=expandcrpinputs(defprob,lossvalue,w,sigma);

if nargin<5
  R=4;  % number of cumulants to solve
end

% Pre-calculate these bits for efficiency
nupowerj=cumprod(repmat(lossvalue,1,R),2);   % NxR matrix
pw=repmat(defprob,1,K1).*w;                     % NxK matrix
s2=sigma.^2;

kappa=zeros(1,R);
for i=1:K1
  Q=pw(:,i)'*nupowerj;
  if sigma(i)==0
    kappa=kappa+Q;
  else
    V=s2(i)*Q;
    for r=1:R
      kappa(r)=kappa(r)+psideriv(V,r)/s2(i);
    end
  end
end

function S=psideriv(V,r)
switch r
case 1,    S=V(1);
case 2,    S=V(2)+V(1)^2;
case 3,    S=V(3)+3*V(2)*V(1)+2*V(1)^3;
case 4,    S=V(4)+4*V(3)*V(1)+3*V(2)^2+12*V(2)*V(1)^2+6*V(1)^4;
case 5,    S=V(5)+5*V(4)*V(1)+10*V(3)*V(2)+20*V(3)*V(1)^2 ...
                 +30*V(2)^2*V(1)+60*V(2)*V(1)^3+24*V(1)^5;
case 6,    S=V(6)+6*V(5)*V(1)+15*V(4)*V(2)+30*V(4)*V(1)^2 ...
                 +10*V(3)^2+120*V(3)*V(2)*V(1)+120*V(3)*V(1)^3 ...
		 +30*V(2)^3+270*V(2)^2*V(1)^2+360*V(2)*V(1)^4 ...
		 +120*V(1)^6;
otherwise,   error('psideriv has run out of derivative formulae!')
end

