function [VaR,summarystats,errorcode]=crplus(defprob,lossvalue,w,sigma,q,lossunit)
%CRPLUS: Matlab implementation of CreditRisk+ 
% [VaR,summarystats,errorcode]=crplus(defprob,lossvalue,w,sigma,q,lossunit)
%
% Inputs:
%    defprob   = default probabilities for obligors
%                Nx1 or scalar (if scalar, assume same for all obligors)
%    lossvalue = book value loss in event of default
%                Nx1 vector (full vector required!!)
%    w     = factor loadings on systematic factors
%            NxK or 1xK (if row vector, assume same for all obligors)
%    sigma = sector volatilities: sigma(k) is standard deviation of X_k
%            1xK vector
%    q     = percentile points for which to obtain Value-at-Risk
%            Mx1 vector
%    lossunit = standard unit of loss
%            If omitted, set so that maximum standardized loss is no
%            greater than 200.
%
% Outputs:
%    VaR   = Value-at-Risk at q-th percentile points
%            Mx1 vector
%    summarystats = [mean stddev skewness kurtosis] of loss distribution
%            4x1 vector
%            Note: Kurtosis(y) defined as E[(y-E[y])^4]/E[(y-E[y])^2]^2
%    errorcode = error status code (scalar)
%                0 = successful calculation
%                1 = negative probability in recurrence algorithm
%                2 = recurrence algorithm appears unable to reach
%                    maximum desired percentile of distribution
%                3 = moments from recurrence algorithm do not match
%                    analytical moments
%
% Notation:
%      N = number of exposures in portfolio
%      K = number of systematic sectors
% Formula for conditional default probability for obligor i is
%      p_i(x) = DefProb_i * (1+ sum_k w_{ik}*(x_k+1))
%                 where underscore indicates subscript 
%                 and summation is for k=1..K
%      Factor loadings w_{ik} sum across k to less than or equal to
%      one.  If less than one, remaining risk is assigned to
%      specific risk sector (see A12.3 in CR+ manual).
% Distribution for X_k is gamma with mean 1, variance sigma_k^2.
%      NOTE!  The CR+ manual has mean of X_k equal to mu_k, but
%      then normalizes out the mu_k in calculating the unconditional
%      loss distribution.  My approach is numerically equivalent.
% Loss exposure in standardized units is
%      nu_i   = ROUND(lossvalue_i / lossunit)
%
% Requires:  crp_a10 (mex file)
%            expandcrpinputs.m

% CreditRisk+ is trademark of Credit Suisse Financial Products.
% See CSFP (1997) "CreditRisk+: A Credit Risk Management Framework."
%
% Version 1.02 of Matlab implementation by Michael Gordy, 7 March 2000.
% Email: <michael.gordy@frb.gov>
% This program is provided free of charge for research use.  It
% does not constitute endorsement of CreditRisk+ by the Federal
% Reserve Board.

% Set maximum tolerance for percent mismatch 
%     between numerical moments and analytical moments.  
maxemptol=[0.05 0.1]';

% Get theoretical cumulants.
kappa=crpcumulants(defprob,lossvalue,w,sigma,4);

% Expand data, as some vectors may have been given as scalars.
[defprob,w,sigma,N0,K,K1]=expandcrpinputs(defprob,lossvalue,w,sigma);

% Make sure q is a column vector.
if nargin<5 | isempty(q)
  error('CRPLUS: q vector is empty!')
else
  q=q(:);
end


% Parameter lossunit is optional.
%      If omitted, set so that maximum standardized loss is no
%      greater than safemaxnu.
safemaxnu=200;
bigloan=max(lossvalue);
if nargin<6
  lossunit=(bigloan/safemaxnu);
elseif (bigloan/lossunit) > 1.05*safemaxnu
  disp('CRPLUS Warning: Maximum nu in portfolio over recommended limit.')
  disp('                You may want to choose higher LossUnit.')
end
nu=round(lossvalue/lossunit);

% Eliminate any nu of size 0.
ee=(nu==0);
if any(ee)
  nu(ee)=[];
  defprob(ee)=[]; w(ee,:)=[];
end  
N=length(nu);      % will have N<N0 if some nu were zero
clear ee

% Now the serious calculations begin.
% These are some handy constants to pre-calculate.
sig2=sigma.^2;
muk=defprob'*w;     % 1xK1
mus2=muk.*sig2; 
deltak=mus2./(1+mus2);  % 1xK1
maxnu=max(nu);

% Form matrix of P_k functions.
%   These are the P_k(z) polynomials defined in equation (65), 
%   section A9.2 of the CR+ manual. The derivative functions P_k'(z)
%   are calculated below.  Following Matlab's standard notation,
%   high order terms in the polynomials come first.
Pk=zeros(K1,maxnu+1);  % initialize
for m=1:maxnu
  ee=(nu==m);
  if any(ee)
    Pk(:,1+maxnu-m)=(w(ee,:)'*defprob(ee))./muk';
  end
end
clear ee
% Form derivative matrix Pprimek
Pprimek=zeros(K1,maxnu);
for k=1:K1
  Pprimek(k,:)=polyder(Pk(k,:));
end

% Sum the K1 rational polynomials in the log derivative of G(z).
%    This is the rational polynomial expression for G'(z)/G(z)
%    in equation (79), section A10.2 of the CR+ manual.
%    The resulting numerator and denominator polynomials are the
%    A(z) and B(z) polynomials in equation (70), section A10.1.
%    Here, these polynomials are called aa and bb, respectively.
%    Note that Matlab polynomial operations assume high order
%    first.  We will "flip" to low order first just before passing
%    these polynomial vectors to the CRP_A10 mex file.
% Also, get log of first term in polynomial expansion of G (lnG0).
aa=0; bb=1;   lnG0=0;  % initialize
for k=1:K1
  if sigma(k)==0
    aa=polyadd(aa,conv(bb,muk(k)*Pprimek(k,:)));
    lnG0=lnG0-muk(k);
  else
    cc=sig2(k)*polyadd(1,-deltak(k)*Pk(k,:));
    aa=polyadd(conv(aa,cc),conv(bb,deltak(k)*Pprimek(k,:)));
    bb=conv(bb,cc);
    % lnG0=lnG0+log(1-deltak(k))/sig2(k);
    lnG0=lnG0-log(1+mus2(k))/sig2(k);  % equivalent to above
  end  
end
aa=fliplr(aa); bb=fliplr(bb);  % low order first
aa=aa/bb(1);  bb=bb/bb(1);     % normalize
clear Pk Pprimek

% Figure out when to stop in recurrence relation algorithm.
maxpct=0.97*max(q)+0.03;         % add a little leeway to max(q)
maxloss=round(1.25*sum(nu));     % give up at this point

% Run the recurrence relation mex program.
[empmoms,VaR,errorcode]=crp_a10(aa,bb,lnG0,q,maxloss,maxpct,lossunit);
if errorcode==1
  warning('CRPLUS: Negative probability in recurrence algorithm.')
elseif errorcode==2
  warning('CRPLUS: Recurrence algorithm unable to reach max(q) percentile.')
end

% Calculate empirical mean and std dev of loss.
%   Make crude adjustment to uncentered moments to account for 
%   unreached tail numbers.  Assume all mass just above VaR of max(q).
meanstd=empmoms(1:2)+(1-max(q))*((max(VaR)+lossunit).^(1:2))';
meanstd(2)=sqrt(meanstd(2)-meanstd(1)^2);

% Get theoretical summary statistics.
% See if first two empirical moments match.
summarystats = [kappa(1) sqrt(kappa(2)) ...
      kappa(3)/kappa(2)^1.5  3+(kappa(4)/kappa(2)^2)]';
staterr=abs(summarystats(1:2)-meanstd)./summarystats(1:2);
if any(staterr>maxemptol)
  errorcode=3;
  disp(repmat('!',1,62))
  disp('Warning!! CRPLUS numerical Mean and Standard Deviation of loss')
  disp('          do not fit well to analytical values.');
  disp(sprintf('\t\t     Mean     StdDev'))
  disp(sprintf('Analytical\t %9.5f %9.5f',summarystats(1:2)'))
  disp(sprintf('Numerical\t %9.5f %9.5f',meanstd'))
  disp(repmat('!',1,62)), disp(' ')
end
clear kappa centmom empmoms statapprox staterr

% Produce summary report
disp(repmat('-',1,60))
disp('CRPLUS: CreditRisk+ Model of Credit Value-at-Risk')
disp(' '), disp('Portfolio and Risk Factor Structure Parameters:')
dstr='%2.0f systematic risk factor(s).';
if K1>K
  dstr=[dstr(1:end-1) ', plus specific risk.'];
end  
disp(sprintf(dstr,K))
disp([int2str(N0) ' obligors, of which ' int2str(N) ' have nu>0.'])
disp(sprintf('LossUnit = %.7f',lossunit))
dstr='Largest loss exposure is %4.0f standardized loss units.';
disp(sprintf(dstr,maxnu))
dstr='LossUnit*Sum(nu) is %5.2f percent of Sum(LossValue).\n';
disp(sprintf(dstr,100*sum(nu)*lossunit/sum(lossvalue)))
disp('Loss distribution summary statistics:')
disp(sprintf('\tMean: \t\t%6.4e',summarystats(1)))
disp(sprintf('\tStdDev:\t\t%6.4e',summarystats(2)))
disp(sprintf('\tSkewness:\t%5.2f',summarystats(3)))
disp(sprintf('\tKurtosis:\t%5.2f (Normal=3)\n',summarystats(4)))
disp('Value-at-Risk:')
disp(sprintf('\t   q\t    VaR'))
disp(sprintf('\t%6.3f\t %7.3e\n',[q*100 VaR]'))

disp(repmat('-',1,60))
% end of crplus function

function p=polyadd(a,b)
%POLYADD(a,b) returns new polynomial which is sum of a and b.
%  a and b need not be same length.
J=max(length(a),length(b));
p=[zeros(1,J-length(a)) a]+[zeros(1,J-length(b)) b];

function mc=centralmoments(m)
%CENTRALMOMENTS(m) returns centralized moments given noncentralized moments.
%    That is, if given m(j)=E(X^j),
%    CENTRALMOMENTS returns vector E((X-E(X))^j) of length(m).
mc=zeros(size(m,1),size(m,2));
mc(1)=m(1);
m1=[1 reshape(m,1,length(m))];
for i=2:length(m)
  c=1; mi=0;
  for j=0:i
    icj=exp(gammaln(i+1)-gammaln(j+1)-gammaln(i-j+1));
    mi=mi+c*icj*m1(i-j+1);
    c=c*(-m(1));
  end
  mc(i)=mi;
end
