function [defprob,w,sigma,N,K,K1]=expandcrpinputs(defprob,lossvalue,w,sigma)
% EXPANDCRPINPUTS expands CreditRisk+ input data to full dimensions.
%    Purpose is to reduce burden on user in entering data when
%    some characteristics of portfolio are homogeneous across obligors.
%
% Inputs:
%    defprob   = unconditional default probabilities for obligors
%                Nx1 or scalar (if scalar, assume same for all obligors)
%    lossvalue = book value loss in event of default
%                Nx1 vector (full vector required!!)
%    w     = factor loadings on systematic factors
%            NxK or 1xK (if row vector, assume same for all obligors)
%    sigma = sector volatilities: sigma(k) is standard deviation of X_k
%            1xK vector
%
% Outputs:
%    defprob   = Nx1 default probabilities for obligors
%    w     = NxK1 factor loadings on all sectors (row sums = 1)
%            NOTE: first column is for specific risk sector (if used)
%    sigma = 1xK1 sector volatilities 
%            NOTE: first element is 0 for specific risk (if used)
%    N     = number of obligors (scalar)
%    K     = number of systematic factors (scalar)
%    K1    = number of sectors (scalar)
%
% Note:
%    If row sums of input w are not all ones, then it is assumed
%    that a "specific risk sector" is needed.  That sector is
%    assigned to the first column.  Matrix w and vector sigma
%    are "filled out" by prepending this column.
%    See CR+ manual, section A12.3 on specific risk.
%
%    This routine used by crplus.m and crpcumulants.m.

% CreditRisk+ is trademark of Credit Suisse Financial Products.
% See CSFP (1997) "CreditRisk+: A Credit Risk Management Framework."
%
% Matlab implementation by Michael Gordy, 1 March 2000.
% Email: <michael.gordy@frb.gov>

% Dimensions of model to solve
N=length(lossvalue);  K=length(sigma);  K1=K;

% Make DEFPROB dimension match
if length(defprob)==1
  defprob=repmat(defprob,N,1); 
end
if length(defprob)~=N
   error('Length of DEFPROB does not match N.')
end
 
% Specific risk? 
wsum=sum(w,2);
if any(wsum~=1)
  w=[1-wsum w];
  sigma=[0 sigma];
  K1=K+1;   % number of sectors including specific risk
end
clear wsum

% Make w dimension match
if size(w,1)==1, w=repmat(w,N,1); end
if size(w,1)~=N
   error('Row count of w does not match N.')
end
